"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var Debugger = require("debug");
var debug = Debugger('akita:stream');
var ChangeStream = (function () {
    function ChangeStream(stream, reducer) {
        var _this = this;
        this.closed = false;
        this._stream = stream;
        this._queue = [];
        this._cache = '';
        this._reducer = reducer;
        this._listeners = {};
        var parseLine = function () {
            var index = _this._cache.indexOf('\n');
            if (index < 0)
                return;
            var line = _this._cache.substr(0, index);
            _this._cache = _this._cache.substr(index + 1);
            if (line) {
                var json = JSON.parse(line);
                if (!json.type || !json.object) {
                    var error = new Error('Invalid change stream data');
                    if (_this.listenerCount('error')) {
                        _this.emit('error', error);
                    }
                    if (_this._reject) {
                        _this._reject(error);
                        _this._resolve = null;
                        _this._reject = null;
                        return;
                    }
                }
                if (_this._reducer) {
                    json.object = _this._reducer(json.object);
                }
                if (_this.listenerCount('change')) {
                    while (_this._queue.length) {
                        _this.emit('change', _this._queue.shift());
                    }
                    _this.emit('change', json);
                }
                if (_this._resolve) {
                    _this._resolve(json);
                    _this._resolve = null;
                    _this._reject = null;
                }
                else if (!_this.listenerCount('change')) {
                    _this._queue.push(json);
                }
            }
            parseLine();
        };
        this._handler = function (data) {
            var string = data.toString();
            debug('receive', string);
            _this._cache += string;
            parseLine();
        };
        if (stream.getReader) {
            this._reader = stream.getReader();
            var read_1 = function (_a) {
                var done = _a.done, value = _a.value;
                _this._handler(value || '');
                if (done) {
                    _this.closed = true;
                }
                _this._reader.read().then(read_1);
            };
            this._reader.read().then(read_1);
        }
        else {
            stream.on('data', this._handler);
        }
    }
    ChangeStream.prototype.on = function (name, fn) {
        if (!this._listeners[name]) {
            this._listeners[name] = [];
        }
        this._listeners[name].push(fn);
        return this;
    };
    ChangeStream.prototype.emit = function (name) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        if (!this._listeners[name])
            return;
        this._listeners[name].forEach(function (fn) {
            fn.apply(null, args);
        });
    };
    ChangeStream.prototype.listenerCount = function (name) {
        return (this._listeners[name] || []).length;
    };
    ChangeStream.prototype.removeListener = function (name, fn) {
        if (!this._listeners[name])
            return;
        this._listeners[name] = this._listeners[name].filter(function (f) { return f !== fn; });
        return this;
    };
    ChangeStream.prototype.removeAllListeners = function (name) {
        delete this._listeners[name];
        return this;
    };
    ChangeStream.prototype.read = function () {
        var _this = this;
        if (this.closed)
            return Promise.reject(new Error('Can not read from closed stream.'));
        if (this._queue.length) {
            return Promise.resolve(this._queue.shift());
        }
        return new Promise(function (resolve, reject) {
            _this._resolve = resolve;
            _this._reject = reject;
        });
    };
    ChangeStream.prototype.cancel = function () {
        if (this.closed)
            return Promise.reject(new Error('Can not cancel closed stream.'));
        debug('cancel watch');
        if (this._stream.cancel) {
            this._stream.cancel();
        }
        else {
            this._stream.destroy();
        }
        this.closed = true;
    };
    return ChangeStream;
}());
exports.default = ChangeStream;
